import { shallowMount, createLocalVue } from "@vue/test-utils";
import Vuex from "vuex";
import ContentOptions from "../../../src/components/LayoutBuilderContentOptions.vue";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const localVue = createLocalVue();
const store = Store(state);

localVue.use(Vuex);

describe("ContentOptions.vue", () => {
  describe("triggering", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = "1";
    const moveContentStub = jest.fn();
    const addContentStub = jest.fn();
    const deleteContentStub = jest.fn();
    const editElementStub = jest.fn();
    let wrapper;

    beforeEach(() => {
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      wrapper.setMethods({
        moveContent: moveContentStub,
        addContent: addContentStub,
        deleteContent: deleteContentStub,
        editElement: editElementStub
      });
    });

    describe("'moved'", () => {
      describe("on MoveToTopButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermovetotopbutton-stub").vm.$emit("move");
          expect(moveContentStub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: 0
          });
        });
      });

      describe("on MoveUpButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermoveupbutton-stub").vm.$emit("move");
          expect(moveContentStub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: indexSectionBlock - 1
          });
        });
      });

      describe("on MoveDownButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermovedownbutton-stub").vm.$emit("move");
          expect(moveContentStub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: indexSectionBlock + 1
          });
        });
      });

      describe("on MoveToBottomButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermovetobottombutton-stub").vm.$emit("move");
          expect(moveContentStub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: wrapper.vm.indexLast
          });
        });
      });
    });

    describe("'add-element'", () => {
      describe("on AddBeforeButton", () => {
        it("should call addContent() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddbeforebutton-stub")
            .vm.$emit("add-element");
          expect(addContentStub).toBeCalledWith({
            indexSection,
            slotId,
            indexSectionBlock
          });
        });
      });

      describe("on AddAfterButton", () => {
        it("should call addContent() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddafterbutton-stub")
            .vm.$emit("add-element");
          expect(addContentStub).toBeCalledWith({
            indexSection,
            slotId,
            indexSectionBlock: indexSectionBlock + 1
          });
        });
      });
    });

    describe("'edited' on EditButton", () => {
      it("should call editElement() with the correct params", () => {
        wrapper.find("layoutbuildereditbutton-stub").vm.$emit("edit");
        expect(editElementStub).toBeCalledWith({
          indexSection,
          slotId,
          indexSectionBlock
        });
      });
    });

    describe("'deleted' on DeleteButton", () => {
      it("should call deleteContent() with the correct params", () => {
        wrapper.find("layoutbuilderdeletebutton-stub").vm.$emit("delete");
        expect(deleteContentStub).toBeCalledWith({
          indexSection,
          slotId,
          indexSectionBlock
        });
      });
    });
  });

  describe("calling indexLast()", () => {
    const indexSection = 1;
    const slotId = "1";
    const indexSectionBlock = 1;

    it("should return the index of the last content block", () => {
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      expect(wrapper.vm.indexLast).toBe(
        store.state.sections[indexSection].content[slotId].length - 1
      );
    });
  });

  describe("Content is the first one and not the last one", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = "1";
    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId,
        html: ""
      },
      store,
      localVue
    });

    it("isFirstContent() should be true", () => {
      expect(wrapper.vm.isFirstContent).toBe(true);
    });

    it("isLastContent() should be false", () => {
      expect(wrapper.vm.isLastContent).toBe(false);
    });
  });

  describe("Content is neither the last nor the first one", () => {
    const indexSection = 1;
    const indexSectionBlock = 1;
    const slotId = "1";
    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId,
        html: ""
      },
      store,
      localVue
    });

    it("isFirstContent() should be false", () => {
      expect(wrapper.vm.isFirstContent).toBe(false);
    });

    it("isLastContent() should be false", () => {
      expect(wrapper.vm.isLastContent).toBe(false);
    });
  });

  describe("Content is the last one and not the first one", () => {
    const indexSection = 1;
    const slotId = "1";
    const indexSectionBlock =
      store.state.sections[indexSection].content[slotId].length - 1;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      wrapper.setMethods({ moveContent: stub });
    });

    it("isFirstContent() should be false", () => {
      expect(wrapper.vm.isFirstContent).toBe(false);
    });

    it("isLastContent() should be true", () => {
      expect(wrapper.vm.isLastContent).toBe(true);
    });
  });

  describe("Content is the first one and the last one", () => {
    const indexSection = 3;
    const slotId = "1";
    const indexSectionBlock = 0;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      wrapper.setMethods({ moveContent: stub });
    });

    it("isFirstContent() should be true", () => {
      expect(wrapper.vm.isFirstContent).toBe(true);
    });

    it("isLastContent() should be true", () => {
      expect(wrapper.vm.isLastContent).toBe(true);
    });
  });

  describe("calling moveContent()", () => {
    it("should commit 'move' to the store with the correct params", () => {
      const indexSection = 1;
      const indexSectionBlock = 0;
      const slotId = "1";
      const to = 0;
      const stub = jest.fn();
      store.commit = stub;

      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      wrapper.vm.moveContent({ indexSection, slotId, indexSectionBlock, to });

      expect(stub).toHaveBeenCalledWith("move", {
        arr: store.state.sections[indexSection].content[slotId],
        to: 0,
        index: indexSectionBlock
      });
    });
  });

  describe("calling isEditable()", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = "1";

    describe("with an editable element", () => {
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: "foo"
        },
        store,
        localVue
      });

      it("should return true", () => {
        expect(wrapper.vm.isEditable).toBe(true);
      });
    });

    describe("with a not editable element", () => {
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      it("should return false", () => {
        expect(wrapper.vm.isEditable).toBe(false);
      });
    });
  });

  describe("lengthContent()", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = "1";

    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId,
        html: ""
      },
      store,
      localVue
    });

    it("should return the amount of content in this section block", () => {
      expect(wrapper.vm.lengthContent).toBe(
        store.state.sections[indexSection].content[slotId].length
      );
    });
  });

  describe("isDraggable()", () => {
    it("should return true if there are more than 1 content", () => {
      const indexSection = 1;
      const indexSectionBlock = 0;
      const slotId = "1";
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      expect(wrapper.vm.isDraggable).toBe(true);
    });

    it("should return false if there is only 1 content", () => {
      const indexSection = 3;
      const indexSectionBlock = 0;
      const slotId = "1";
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId,
          html: ""
        },
        store,
        localVue
      });

      expect(wrapper.vm.isDraggable).toBe(false);
    });
  });

  describe("calling editElement()", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = "1";
    const id = "foo";
    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId,
        html: ""
      },
      mocks: {
        $root: {
          $el: {
            id
          }
        }
      },
      store,
      localVue,
      attachToDocument: true
    });
    const params = {
      indexSection: 1,
      indexSectionBlock: 0,
      slotId: "1"
    };

    it("commits setCurrentlyEditableContent on the store with the given data", () => {
      const stub = jest.fn();
      const stub2 = jest.fn();
      store.commit = stub;
      store.state.adapters.callback = stub2;
      wrapper.vm.editElement(params);
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", params);
    });

    it("calls this.$store.state.adapters.callback() with the root element id and the params object of the element stored in the state", () => {
      const stub = jest.fn();
      store.state.adapters.callback = stub;
      wrapper.vm.editElement(params);
      expect(stub).toHaveBeenCalledWith(
        wrapper.vm.$root.$el.id,
        store.state.elements[
          store.state.sections[params.indexSection].content[params.slotId][
            params.indexSectionBlock
          ]
        ].params
      );
    });
  });
});
