import { shallowMount, createLocalVue } from "@vue/test-utils";
import Vuex from "vuex";
import PageLayoutForm from "../../../src/components/LayoutBuilderPageLayoutForm.vue";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const localVue = createLocalVue();
const store = Store(state);

localVue.use(Vuex);

describe("PageLayoutForm.vue", () => {
  const action = "foo";

  const wrapper = shallowMount(PageLayoutForm, {
    propsData: {
      indexSection: 2,
      section: {
        layout: null
      }
    },
    store,
    localVue
  });

  const stub = jest.fn();
  store.dispatch = stub;

  it("should render the correct form action", () => {
    expect(wrapper.element.action).toEqual(action);
  });

  it("sets the stringified value of state.sections as the value of the section input", () => {
    expect(
      wrapper.find(".LayoutBuilderPageLayoutForm-input").attributes().value
    ).toEqual(JSON.stringify(store.state.sections));
  });

  describe("submitting the form", () => {
    wrapper.vm.save = stub;

    it("calls save()", () => {
      wrapper.element.dispatchEvent(new Event("submit"));
      expect(stub).toHaveBeenCalled();
    });
  });

  describe("calling save()", () => {
    const successStub = jest.fn();
    const errorStub = jest.fn();
    const app = shallowMount(PageLayoutForm, {
      store,
      localVue
    });

    app.vm.error = errorStub;
    app.vm.success = successStub;
    app.vm.handleResponse = stub;

    it("makes a request", () => {
      window.fetch = jest.fn(() => new Promise(resolve => resolve()));
      app.vm.save();
      expect(window.fetch).toHaveBeenCalled();
    });

    describe("successful request", () => {
      window.fetch = jest.fn(
        () =>
          new Promise(resolve =>
            resolve({
              ok: true,
              json: () =>
                new Promise(res =>
                  res({
                    ok: true,
                    success: true
                  })
                )
            })
          )
      );

      it("calls handleResponse()", () => {
        app.vm.save();
        expect(stub).toHaveBeenCalled();
      });
    });
  });

  describe("calling handleResponse()", () => {
    const app = shallowMount(PageLayoutForm, {
      store,
      localVue
    });
    const successStub = jest.fn();
    const errorStub = jest.fn();
    app.vm.success = successStub;
    app.vm.error = errorStub;

    describe("with success:true", () => {
      it("calls success() with the given message", done => {
        app.vm.handleResponse({
          ok: true,
          json: () =>
            new Promise(res =>
              res({
                success: true,
                message: "success"
              })
            )
        });

        app.vm.$nextTick(() => {
          expect(successStub).toHaveBeenCalledWith("success");
          done();
        });
      });
    });

    describe("with success:false", () => {
      it("calls error() with the given message", done => {
        app.vm.handleResponse({
          ok: true,
          json: () =>
            new Promise(res =>
              res({
                success: false,
                message: "error"
              })
            )
        });

        app.vm.$nextTick(() => {
          expect(errorStub).toHaveBeenCalledWith("error");
          done();
        });
      });
    });

    describe("with a not okayish response", () => {
      it("calls error()", done => {
        app.vm.handleResponse({
          ok: false
        });

        app.vm.$nextTick(() => {
          expect(errorStub).toHaveBeenCalled();
          done();
        });
      });
    });

    describe("with a not parseable response", () => {
      it("calls error()", done => {
        app.vm.handleResponse({
          ok: true,
          json: () => new Promise((resolve, reject) => reject())
        });

        app.vm.$nextTick(() => {
          expect(errorStub).toHaveBeenCalled();
          done();
        });
      });
    });
  });

  describe("calling success()", () => {
    it("sets this.message to the given value", () => {
      const message = "success";
      wrapper.vm.success(message);
      expect(wrapper.vm.message).toBe(message);
    });

    it("sets this.messageType to the 'success'", () => {
      wrapper.vm.success();
      expect(wrapper.vm.messageType).toBe("success");
    });
  });

  describe("calling error()", () => {
    it("sets this.message to the given value", () => {
      const message = "error";
      wrapper.vm.error(message);
      expect(wrapper.vm.message).toBe(message);
    });

    it("sets this.messageType to the 'error'", () => {
      wrapper.vm.error();
      expect(wrapper.vm.messageType).toBe("error");
    });
  });

  describe("calling removeMessage()", () => {
    it("sets this.message to false", () => {
      wrapper.vm.message = "foo";
      wrapper.vm.removeMessage();
      expect(wrapper.vm.message).toBe(false);
    });
  });
});
