import { shallowMount, createLocalVue } from "@vue/test-utils";
import Vuex from "vuex";
import SectionOptions from "../../../src/components/LayoutBuilderSectionOptions.vue";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const localVue = createLocalVue();
const store = Store(state);

localVue.use(Vuex);

describe("SectionOptions.vue", () => {
  describe("triggering", () => {
    const indexSection = 2;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection,
          section: {
            layout: "LayoutOneColumn"
          }
        },
        store,
        localVue
      });

      wrapper.setMethods({
        moveSection: stub,
        addSection: stub,
        deleteSection: stub
      });
    });

    describe("'moved'", () => {
      describe("on MoveToTopButton", () => {
        it("should call moveSection() with the correct params", () => {
          wrapper.find("layoutbuildermovetotopbutton-stub").vm.$emit("move");
          expect(stub).toBeCalledWith({
            indexSection,
            to: 0
          });
        });
      });

      describe("on MoveUpButton", () => {
        it("should call moveSection() with the correct params", () => {
          wrapper.find("layoutbuildermoveupbutton-stub").vm.$emit("move");
          expect(stub).toBeCalledWith({
            indexSection,
            to: indexSection - 1
          });
        });
      });

      describe("on MoveDownButton", () => {
        it("should call moveSection() with the correct params", () => {
          wrapper.find("layoutbuildermovedownbutton-stub").vm.$emit("move");
          expect(stub).toBeCalledWith({
            indexSection,
            to: indexSection + 1
          });
        });
      });

      describe("on MoveToBottomButton", () => {
        it("should call moveSection() with the correct params", () => {
          wrapper.find("layoutbuildermovetobottombutton-stub").vm.$emit("move");
          expect(stub).toBeCalledWith({
            indexSection,
            to: wrapper.vm.indexLast
          });
        });
      });
    });

    describe("'add-element'", () => {
      describe("on AddBeforeButton", () => {
        it("should call addSection() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddbeforebutton-stub")
            .vm.$emit("add-element");
          expect(stub).toBeCalledWith(indexSection);
        });
      });

      describe("on AddAfterButton", () => {
        it("should call addSection() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddafterbutton-stub")
            .vm.$emit("add-element");
          expect(stub).toBeCalledWith(indexSection + 1);
        });
      });
    });

    describe("'deleted' on DeleteButton", () => {
      it("should call deleteSection() with the correct params", () => {
        wrapper.find("layoutbuilderdeletebutton-stub").vm.$emit("delete");
        expect(stub).toBeCalledWith(indexSection);
      });
    });
  });

  describe("calling indexLast()", () => {
    it("should return the index of the last section", () => {
      const wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection: 0,
          section: {
            layout: "LayoutOneColumn"
          }
        },
        store,
        localVue
      });

      expect(wrapper.vm.indexLast).toBe(store.state.sections.length - 1);
    });
  });

  describe("Section is the first one and not the last one", () => {
    const indexSection = 0;
    const wrapper = shallowMount(SectionOptions, {
      propsData: {
        indexSection,
        section: {
          layout: "LayoutOneColumn"
        }
      },
      store,
      localVue
    });

    it("isFirstSection() should be true", () => {
      expect(wrapper.vm.isFirstSection).toBe(true);
    });

    it("isLastSection() should be false", () => {
      expect(wrapper.vm.isLastSection).toBe(false);
    });
  });

  describe("Section is neither the last nor the first one", () => {
    const indexSection = 2;
    const wrapper = shallowMount(SectionOptions, {
      propsData: {
        indexSection,
        section: {
          layout: "LayoutOneColumn"
        }
      },
      store,
      localVue
    });

    it("isFirstSection() should be false", () => {
      expect(wrapper.vm.isFirstSection).toBe(false);
    });

    it("isLastSection() should be false", () => {
      expect(wrapper.vm.isLastSection).toBe(false);
    });
  });

  describe("Section is the last one and not the first one", () => {
    const indexSection = store.state.sections.length - 1;
    let wrapper;

    beforeEach(() => {
      wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection,
          section: {
            layout: "LayoutOneColumn"
          }
        },
        store,
        localVue
      });
    });

    it("isFirstSection() should be false", () => {
      expect(wrapper.vm.isFirstSection).toBe(false);
    });

    it("isLastSection() should be true", () => {
      expect(wrapper.vm.isLastSection).toBe(true);
    });
  });

  describe("Section is the only one", () => {
    const indexSection = 0;

    describe("", () => {
      let wrapper;

      const newStore = Store({
        sections: [
          {
            id: Date.now(),
            content: {},
            layout: null
          }
        ]
      });

      beforeEach(() => {
        wrapper = shallowMount(SectionOptions, {
          propsData: {
            indexSection,
            section: newStore.state.sections[indexSection]
          },
          store: newStore,
          localVue
        });
      });

      it("isFirstSection() should be true", () => {
        expect(wrapper.vm.isFirstSection).toBe(true);
      });

      it("isLastSection() should be true", () => {
        expect(wrapper.vm.isLastSection).toBe(true);
      });

      describe("section has no content and no layout", () => {
        it("isDeletable() should be false", () => {
          expect(wrapper.vm.isDeletable).toBe(false);
        });
      });
    });

    describe("section has content or layout", () => {
      const newStore = Store({
        sections: [
          {
            id: Date.now(),
            content: { 1: [1] },
            layout: "Foo"
          }
        ]
      });

      const wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection,
          section: newStore.state.sections[indexSection]
        },
        store: newStore,
        localVue
      });

      it("isDeletable() should be true", () => {
        expect(wrapper.vm.isDeletable).toBe(true);
      });
    });
  });

  describe("calling moveSection()", () => {
    it("should commit 'move' to the store with the correct params", () => {
      const indexSection = 1;
      const to = 0;
      const stub = jest.fn();
      store.commit = stub;

      const wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection,
          section: {
            layout: "LayoutOneColumn"
          }
        },
        store,
        localVue
      });

      wrapper.vm.moveSection({ indexSection, to });

      expect(stub).toHaveBeenCalledWith("move", {
        arr: store.state.sections,
        to: 0,
        index: indexSection
      });
    });
  });

  describe("section is not the only one", () => {
    const indexSection = store.state.sections.length - 1;
    let wrapper;

    beforeEach(() => {
      wrapper = shallowMount(SectionOptions, {
        propsData: {
          indexSection,
          section: {
            layout: "LayoutOneColumn"
          }
        },
        store,
        localVue
      });
    });

    it("isDeletable() should be true", () => {
      expect(wrapper.vm.isDeletable).toBe(true);
    });
  });
});
